import json
import pytest
from unittest.mock import patch

from fastmcp import Client
from mvf1.mcp import mcp


with open('tests/players.json') as f:
    mock_players = json.load(f)


def configure_mock_response(mock_urlopen, payload):
    if isinstance(payload, Exception):
        mock_urlopen.side_effect = payload
    else:
        mock_urlopen.return_value = payload


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_f1_live_timing_clock(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {"data": {"liveTimingClock": None}})
        response = await client.call_tool('f1_live_timing_clock')
        mock_urlopen.assert_called_once()
        assert "liveTimingClock" in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_f1_live_timing_state(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'liveTimingState': None}})
        response = await client.call_tool('f1_live_timing_state')
        mock_urlopen.assert_called_once()
        assert "liveTimingState" in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_fiawec_live_timing_state(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'liveTimingState': None}})
        response = await client.call_tool("fiawec_live_timing_state")
        mock_urlopen.assert_called_once()
        assert "liveTimingState" in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_players(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                mock_players)
        response = await client.call_tool("players")
        mock_urlopen.assert_called_once()
        assert response[0].text.count(":") == 2
        assert "INTERNATIONAL" in response[0].text


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_system_info(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data':
                                    {'systemInfo':
                                        {'platform': 'linux', 'arch': 'x64'}}})
        response = await client.call_tool("system_info")
        mock_urlopen.assert_called_once()
        assert 'systemInfo' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_version(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'version': '1.12.6'}})
        response = await client.call_tool("version")
        mock_urlopen.assert_called_once()
        assert 'version' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data':
                                    {'player':
                                        mock_players['data']['players'][0]}})
        response = await client.call_tool("player", {"id": 3})
        mock_urlopen.assert_called_once()
        assert "INTERNATIONAL" in response[0].text


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_create(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerCreate': '3'}})
        stream_data = mock_players['data']['players'][0]['streamData']
        response = await client.call_tool("player_create",
                                          {"content_id":
                                              stream_data['contentId']})
        assert mock_urlopen.call_count == 1
        assert "playerCreate" in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_create_with_bounds(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerCreate': '3'}})
        stream_data = mock_players['data']['players'][0]['streamData']
        response = await client.call_tool("player_create",
                                          {"content_id":
                                              stream_data['contentId'],
                                           "x": 0,
                                           "y": 0,
                                           "width": 640,
                                           "height": 480})
        assert mock_urlopen.call_count == 1
        assert "playerCreate" in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_sync_to_commentary(mock_urlopen):
    async with Client(mcp) as client:
        mock_urlopen.side_effect = [
            mock_players,
            {'data': {'playerSync': True}}
        ]
        response = await client.call_tool("player_sync_to_commentary")
        assert mock_urlopen.call_count == 2
        assert 'Sync' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_delete(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerDelete': True}})
        response = await client.call_tool("player_delete",
                                          {"id": 3})

        mock_urlopen.assert_called_once()
        assert 'playerDelete' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_seek_to(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSeekTo': 0}})
        response = await client.call_tool("player_seek_to",
                                          {"id": 3,
                                           "absolute": 0})
        mock_urlopen.assert_called_once()
        assert 'SeekTo' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_set_bounds(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data':
                                    {'playerSetBounds':
                                        {'x': 0,
                                         'y': 0,
                                         'width': 720,
                                         'height': 408}}})
        response = await client.call_tool("player_set_bounds",
                                          {"id": 3,
                                           "x": 0,
                                           "y": 0})
        mock_urlopen.assert_called_once()
        assert 'SetBounds' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_set_always_on_top(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerAlwaysOnTop': True}})
        response = await client.call_tool("player_set_always_on_top",
                                          {"id": 3})
        mock_urlopen.assert_called_once()
        assert 'AlwaysOnTop' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_set_driver_header_mode(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data':
                                    {'playerSetDriverHeaderMode':
                                        'OBC_LIVE_TIMING'}})
        response = await client.call_tool("player_set_driver_header_mode",
                                          {"id": 3,
                                           "mode": "OBC_LIVE_TIMING"})

        mock_urlopen.assert_called_once()
        assert 'OBC_LIVE_TIMING' in str(response)

@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_set_volume(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerVolume': 0}})
        response = await client.call_tool("player_set_volume",
                                          {"id": 3,
                                           "volume": 0})
        mock_urlopen.assert_called_once()
        assert 'Volume' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_set_paused(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerPaused': True}})
        response = await client.call_tool("player_set_paused",
                                          {"id": 3,
                                           "paused": True})
        mock_urlopen.assert_called_once()
        assert 'Paused' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_set_fullscreen(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerFullscreen': True}})
        response = await client.call_tool("player_set_fullscreen",
                                          {"id": 3,
                                           "fullscreen": True})
        mock_urlopen.assert_called_once()
        assert 'Fullscreen' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_set_muted(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerMuted': True}})
        response = await client.call_tool("player_set_muted",
                                          {"id": 3,
                                           "muted": True})
        mock_urlopen.assert_called_once()
        assert 'Muted' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_set_speedometer_visibility(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSpeedometerVisible': True}})
        response = await client.call_tool("player_set_"
                                          "speedometer_visibility",
                                          {"id": 3,
                                           "visible": True})
        mock_urlopen.assert_called_once()
        assert 'Visible' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_sync(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSynced': True}})
        response = await client.call_tool("player_sync",
                                          {"id": 3})
        mock_urlopen.assert_called_once()
        assert 'Synced' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_get_content_id(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                mock_players)
        response = await client.call_tool("player_get_content_id")
        mock_urlopen.assert_called_once()
        assert '1000001067' in str(response)


@pytest.mark.asyncio
@patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
async def test_player_get_content_id_no_players(mock_urlopen):
    async with Client(mcp) as client:
        configure_mock_response(mock_urlopen,
                                {"data": {"players": []}})
        response = await client.call_tool("player_get_content_id")
        mock_urlopen.assert_called_once()
        assert 'No players' in str(response)



from typing import Optional

from fastmcp import FastMCP

from mvf1 import MultiViewerForF1
from mvf1.mvf1_schema import Player

client = MultiViewerForF1()

mcp = FastMCP(name="MultiViewer MCP Server",
              instructions="""
              This server provides control of MultiViewer,
              a desktop application for watching motorsports
              series like Formula 1, World Endurance Championship,
              IndyCar, Formula 2, Formula 3 and Formula 1 Academy.

              It is useful app as it provides the viewer with the
              ability to run multiple video streams from the same
              event successfully including the main broadcast,
              driver onboards and data on the event.

              Call f1_live_timing_state() for all the data about
              the Formula 1 racing session so far.

              Call wec_live_timing_state() for all the data about
              the World Endurance Championship racing sesion
              so far.

              Call players() to get the ideas and titles for all
              the currently running players.

              Call the tools prefixed with players_ to create,
              delete or manipulate existing MultiViewer video
              players.

              For most prompts, you will want to call the appropriate
              live_timing tool first to get the current state
              of the racing session (F1 or WEC only) and then call
              players() to get the current state of all the video
              players.

              Note that new players take between 500-800 milliseconds
              to full initialize - always pause before performing
              a mutation on a freshly created player.

              Also note that when a video is fullscreen it should be the
              first player deployed so that subsequent players will appear
              on top of it, creating a picture-in-picture effect.

              Driver onboards laid on top of a fullscreen should be no
              wider than one-fifth of the screen's width. Multiple
              driver onboards should be tiled on the left side
              of the screen.
              """)


@mcp.tool()
def f1_live_timing_clock() -> dict:
    """
    Returns the time for a Formula 1 event when it is playing.

    Returns
    -------
    dict
        Current time.

    """

    return client.f1_live_timing_clock


@mcp.tool()
def f1_live_timing_state() -> dict:
    """
    Returns state of live timing for a Formula 1 event
    when it is playing. These data provide the current
    race information including racing order, lap times,
    race control messages, position on track, and
    driver telemetry.

    Returns
    -------
    dict
        Current state of Formula 1 race.

    """

    return client.f1_live_timing_state


@mcp.tool()
def fiawec_live_timing_state() -> dict:
    """
    Returns state of live timing for a World Endurance
    Championship event when it is playing. These data
    provide the current race information including
    racing order, lap times, race control messages,
    position on track, and driver telemetry.

    Returns
    -------
    dict
        Current state of World Endurance Championship
        race.

    """

    return client.fiawec_live_timing_state


@mcp.tool()
def players() -> list:
    """
    Returns a list of active MultiViewer video players
    displaying content.

    Returns
    -------
    list
        List of Player objects.

    """

    return client.players


@mcp.tool()
def system_info() -> dict:
    """
    Returns the information on the system running MultiViewer.

    Returns
    -------
    dict
        System information.

    """

    return client.system_info


@mcp.tool()
def version() -> dict:
    """
    Returns the MultiViewer version.

    Returns
    -------
    dict
        Version information.

    """

    return client.version


@mcp.tool()
def player(id: int) -> Player:
    """
    Returns the MultiViewer video player with specific id.

    Parameters
    ----------
    id: int
        Id of video player.

    Returns
    -------
    dict
        Player object.

    """

    return client.player(id=id)


@mcp.tool()
def player_create(
    content_id: int,
    driver_tla: Optional[str] = None,
    driver_number: Optional[int] = None,
    stream_title: Optional[str] = None,
    x: Optional[int] = None,
    y: Optional[int] = None,
    width: Optional[int] = None,
    height: Optional[int] = None,
    fullscreen: Optional[bool] = False,
    always_on_top: Optional[bool] = False,
    maintain_aspect_ratio: Optional[bool] = True,
) -> dict:
    """
    Creates a new MultiViewer video player.

    Parameters
    ----------
    content_id: int
        The contentId for the event.

    driver_tla: str, optional
        Driver three letter acronym (e.g. 'PER' or 'HAM').

    driver_number: int, optional
        Driver Number.

    stream_title: str, optional
        Stream Title.

    x: int, optional
        X coordinate of player's top left corner.

    y: int, optional
        Y coordinate of player's top left corner.

    width: int, optional
        Width of player.

    height: int, optional
        Height of player.

    fullscreen: bool, optional
        Fullscreen status.

    always_on_top: bool, optional
        Always on top status.

    maintain_aspect_ratio: bool, optional
        Maintain aspect ratio status.

    Returns
    -------
    dict
        Dict with result of playerCreate operation. If successful, will
        include the `player_id`.

        Implementation note: we return a `player_id` instead of a `Player`
        object is that there is ~200-800 millisecond lag in between the
        invocation of `player_create` and the `player_id` being accessible
        via the MultiViewer For F1 GraphQL API.  Player creation is
        non-blocking and there is no callback when it is complete. As a
        consequence, we leave this as an implmentation decision for you.

        Most folks put a sleep in between player creation and player
        access.
    """

    return client.player_create(content_id=content_id,
                                driver_tla=driver_tla,
                                driver_number=driver_number,
                                stream_title=stream_title,
                                x=x,
                                y=y,
                                width=width,
                                height=height,
                                fullscreen=fullscreen,
                                always_on_top=always_on_top,
                                maintain_aspect_ratio=maintain_aspect_ratio)


@mcp.tool()
def player_delete(id: int) -> dict:
    """
    Deletes a MultiViewer video player.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player.

    Returns
    -------
    dict
        Deletion response.

    """

    return client.player_delete(id=id)


@mcp.tool()
def player_seek_to(id: int,
                   absolute: Optional[int] = None,
                   relative: Optional[int] = None) -> dict:
    """
    Seeks to a specific position in time for a MultiViewer view player.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player.

    absolute: int, optional
        Absolute position in the timeline.

    relative: int, optional
        Relative position from current..

    Returns
    -------
    dict
        Seek response.

    """

    return client.player_seek_to(id=id,
                                 absolute=absolute,
                                 relative=relative)


@mcp.tool()
def player_set_bounds(
    id: int,
    x: Optional[int] = None,
    y: Optional[int] = None,
    width: Optional[int] = None,
    height: Optional[int] = None,
) -> dict:
    """
    Set the bounds and/or position of a MultiViewer video player.

    Parameters
    ----------
    id: int
        Id of a MultiViewer video player.

    x: int, optional
        X coordinate of player's top left corner.

    y: int, optional
        Y coordinate of player's top left corner.

    width: int, optional
        Width of player.

    height: int, optional
        Height of player.

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_set_bounds(id=id,
                                    x=x,
                                    y=y,
                                    width=width,
                                    height=height)


@mcp.tool()
def player_set_volume(id: int, volume: int) -> dict:
    """
    Set the volume of a MultiViewer video player.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player.

    volume: int
        Volume of player from 1 to 100.

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_set_volume(id=id, volume=volume)


@mcp.tool()
def player_set_paused(id: int, paused: Optional[bool] = None) -> dict:
    """
    Pauses/unpauses a MultiViewer video player or
    specifies pause state for player.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player.

    paused: bool, optional
        Desired pause state for MultiViewer video player.

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_set_paused(id=id, paused=paused)


@mcp.tool()
def player_set_fullscreen(id: int, fullscreen: Optional[bool] = None) -> dict:
    """
    Toggles fullscreen for a MultiViewer video player
    or specifies fullscreen state for player.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player.

    fullscreen: bool, optional
        Desired fullscreen state of player.

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_set_fullscreen(id=id, fullscreen=fullscreen)


@mcp.tool()
def player_set_muted(id: int, muted: Optional[bool] = None) -> dict:
    """
    Mutes/unmutes MultiViewer video player or specifies
    muted state for player.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player.

    muted: bool, optional
        Desired muted state of player.

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_set_muted(id=id, muted=muted)


@mcp.tool()
def player_set_speedometer_visibility(
    id: int, visible: Optional[bool] = None
) -> dict:
    """
    Makes speedometer overlay on MultiViewer player
    visible/invisible or specifies visibility.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player.

    visible: bool, optional
        Visibility state of speedometer of player.

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_set_speedometer_visibility(id=id,
                                                    visible=visible)


@mcp.tool()
def player_set_driver_header_mode(id: int,
                                  mode: Optional[str] = None) -> dict:

    """
    Sets the overlay display for a driver onboard MultiViewer vide
    player.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player.

    mode: str, optional
        Desired overlay of the player - can be DRIVER_HEADER, NONE or
        OBC_LIVE_TIMING

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_set_driver_header_mode(id=id,
                                                mode=mode)


@mcp.tool()
def player_set_always_on_top(
    id: int, always_on_top: Optional[bool] = None
) -> dict:
    """
    Sets a MultiPlayer video player on/off always on top.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player.

    always_on_top: bool, optional
        Is the player always on top?

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_set_always_on_top(id=id,
                                           always_on_top=always_on_top)


@mcp.tool()
def player_sync(id: int) -> dict:
    """
    Synchronizes all players to the timestamp of specified MultiViewer
    video player.

    Parameters
    ----------
    id: int
        Id of MultiViewer video player with which to
        sync all other players.

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_sync(id=id)


@mcp.tool()
def player_sync_to_commentary() -> dict:
    """
    Synchronizes all MultiViewer video players to the player
    with a broadcast commentary stream.

    Returns
    -------
    dict
        True if operation is successful.

    """

    return client.player_sync_to_commentary()


@mcp.tool()
def player_get_content_id() -> str:
    """
    Retrieve the content_id from the first MultiViewer video player.

    Returns
    -------
    str
        The content_id of the currently playing session.

    """

    players = client.players

    if len(players) > 0:
        return players[0].content_id
    else:
        return "No players are currently active."



import click

from mvf1 import MultiViewerForF1
from mvf1 import MultiViewerForF1Error
from mvf1 import __version__
from mvf1 import mcp

from urllib.error import URLError

remote = MultiViewerForF1()


@click.group()
@click.version_option(__version__)
def cli():
    pass


@cli.group(name="players", help="Query and control MultiViewerForF1 players.")
def mv_players():
    pass


@mv_players.command(help="List all active MultiViewerForF1 players.")
@click.option(
    "--verbose", is_flag=True, default=False, help="Display all player information."
)
def ls(verbose):
    try:
        players = remote.players
    except URLError:
        raise click.UsageError("MultiViewer for F1 is not found. Is the app " "open?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")

    if len(players) == 0:
        raise click.UsageError("No active players found. Is MultiViewerForF1 running?")

    for player in players:
        click.echo(f"ID: {player.id} - Title: {player.title}")
        if verbose:
            click.echo("--------------------------------------------------")
            click.echo(f"State: {player.state}")
            click.echo(f"driverData: {player.driver_data}")
            click.echo(f"streamData: {player.stream_data}")
            click.echo(f"Position: {player.x}, {player.y}")
            click.echo(f"Dimensions: {player.width}, {player.height}")
            click.echo(f"Fullscreen: {player.fullscreen}")
            click.echo(f"Always On Top: {player.always_on_top}")
            click.echo(f"Maintain Aspect Ratio: {player.maintain_aspect_ratio}")


@mv_players.command(help="Close all active MultiViewerForF1 players.", name="close")
def players_close():
    try:
        players = remote.players
    except URLError:
        raise click.UsageError("MultiViewer for F1 is not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")

    for player in players:
        click.echo(f"Closing ID {player.id} - {player.title}...")
        try:
            player.delete()
        except URLError:
            raise click.UsageError("MultiViewer for F1 not found. Is the app "
                                   "running?")
        except MultiViewerForF1Error as e:
            raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
        except Exception as e:
            raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_players.command(help="Mute all active MultiViewerForF1 players.", name="mute")
def players_mute():
    try:
        players = remote.players
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    for player in players:
        click.echo(f"Muting ID {player.id} - {player.title}...")
        try:
            player.mute()
        except URLError:
            raise click.UsageError("MultiViewer for F1 not found. Is the app "
                                   "running?")
        except MultiViewerForF1Error as e:
            raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
        except Exception as e:
            raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_players.command(help="Pause all active MultiViewerForF1 players.", name="pause")
def players_pause():
    try:
        players = remote.players
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    for player in players:
        click.echo(f"Pausing ID {player.id} - {player.title}...")
        try:
            player.pause()
        except URLError:
            raise click.UsageError("MultiViewer for F1 not found. Is the app "
                                   "running?")
        except MultiViewerForF1Error as e:
            raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
        except Exception as e:
            raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_players.command(help="Toggle speedometers for driver feeds.",
                    name="toggle-speedometers")
def players_set_speedometer_visibility():
    try:
        players = remote.players
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    for player in players:
        try:
            if len(player.title) == 3:
                click.echo(f"Toggling speedometer for ID {player.id} - {player.title}...")
                player.set_speedometer_visibility()
        except URLError:
            raise click.UsageError("MultiViewer for F1 not found. Is the app "
                                   "running?")
        except MultiViewerForF1Error as e:
            raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
        except Exception as e:
            raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_players.command(
    help="Synchronize all active MultiViewerForF1 players to the "
    "player streaming broadcast commentary.",
    name="sync",
)
def players_sync():
    click.echo("Syncing all players to broadcast commentary...")
    try:
        remote.player_sync_to_commentary()
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@cli.group(name="player", help="Query and control a specific MultiViewerForF1 player.")
@click.option("--id", required=False, type=int)
@click.option("--title", required=False, type=str)
@click.pass_context
def mv_player(ctx, id, title):
    ctx.ensure_object(dict)

    if id:
        try:
            ctx.obj["player"] = remote.player(id)
        except URLError:
            raise click.UsageError("MultiViewer for F1 not found. Is the app "
                                   "running?")
        except MultiViewerForF1Error as e:
            raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
        except Exception as e:
            raise click.UsageError(f"Unexpected error: {str(e)}")
    else:
        try:
            players = remote.players
        except URLError:
            raise click.UsageError("MultiViewer for F1 not found. Is the app "
                                   "running?")
        except MultiViewerForF1Error as e:
            raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
        except Exception as e:
            raise click.UsageError(f"Unexpected error: {str(e)}")

        for player in players:
            if player.title == title:
                ctx.obj["player"] = player
                return
        raise click.UsageError("No player found with title " f"{title}")


@mv_player.command(help="Display information about player.")
@click.option(
    "--verbose", is_flag=True, default=False, help="Display all player information."
)
@click.pass_context
def query(ctx, verbose):
    player = ctx.obj["player"]

    click.echo(f"ID: {player.id} - Title: {player.title}")
    if verbose:
        click.echo("--------------------------------------------------")
        click.echo(f"State: {player.state}")
        click.echo(f"driverData: {player.driver_data}")
        click.echo(f"streamData: {player.stream_data}")
        click.echo(f"Position: {player.x}, {player.y}")
        click.echo(f"Dimensions: {player.width}, {player.height}")
        click.echo(f"Fullscreen: {player.fullscreen}")
        click.echo(f"Always On Top: {player.always_on_top}")
        click.echo(f"Maintain Aspect Ratio: {player.maintain_aspect_ratio}")


@mv_player.command(help="Close player", name="close")
@click.pass_context
def player_close(ctx):
    player = ctx.obj["player"]
    click.echo(f"Closing player ID: {player.id} - {player.title}...")
    try:
        player.delete()
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_player.command(help="Seek to relative or absolute position")
@click.option("--absolute", required=False, type=int)
@click.option("--relative", required=False, type=int)
@click.pass_context
def seek(ctx, absolute, relative):
    if not absolute and not relative:
        raise click.UsageError(
            "To seek a player, pass with an --absolute or "
            "--relative number of seconds. This can any positive"
            "or negative integer."
        )

    player = ctx.obj["player"]
    click.echo(f"Seeking player ID {player.id} - {player.title}...")
    try:
        player.seek(absolute=absolute, relative=relative)
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_player.command(help="Adjust position and/or dimensions")
@click.option("--x", required=False, type=int)
@click.option("--y", required=False, type=int)
@click.option("--width", required=False, type=int)
@click.option("--height", required=False, type=int)
@click.pass_context
def set_bounds(ctx, x, y, width, height):
    player = ctx.obj["player"]
    click.echo(f"Adjusting bounds of player ID {player.id} - " f"{player.title}...")
    try:
        player.set_bounds(x=x, y=y, width=width, height=height)
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_player.command(help="Adjust volume")
@click.option("--volume", required=True, type=int)
@click.pass_context
def set_volume(ctx, volume):
    player = ctx.obj["player"]
    click.echo(f"Setting volume for player ID {player.id} - " f"{player.title}...")
    try:
        player.set_volume(volume=volume)
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_player.command(help="Pause")
@click.pass_context
def pause(ctx):
    player = ctx.obj["player"]
    click.echo(f"Pausing player ID {player.id} - {player.title}...")
    try:
        player.pause()
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_player.command(help="Toggle fullscreen")
@click.pass_context
def set_fullscreen(ctx):
    player = ctx.obj["player"]
    click.echo(f"Toggling fullscreen for player ID {player.id} - " f"{player.title}...")
    try:
        player.set_fullscreen()
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_player.command(help="Mute")
@click.pass_context
def mute(ctx):
    player = ctx.obj["player"]
    click.echo(f"Muting player ID {player.id} - {player.title}...")
    try:
        player.mute()
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")

@mv_player.command(help="Toggle always on top")
@click.pass_context
def set_always_on_top(ctx):
    player = ctx.obj["player"]
    click.echo(f"Muting player ID {player.id} - {player.title}...")
    try:
        player.set_always_on_top()
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_player.command(help="Toggle speedometer visibility")
@click.pass_context
def set_speedometer_visibility(ctx):
    player = ctx.obj["player"]
    click.echo(
        f"Toggling speedometer visibility player ID {player.id} - " f"{player.title}..."
    )
    try:
        player.set_speedometer_visibility()
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_player.command(help="Synchronize all players to this player's timestamp")
@click.pass_context
def sync(ctx):
    player = ctx.obj["player"]
    click.echo(
        f"Synchronizing all players to player ID {player.id} - " f"{player.title}..."
    )
    try:
        player.sync()
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@mv_player.command(help="Switch player's video stream.")
@click.option("--title", required=True, type=str)
@click.pass_context
def switch_stream(ctx, title):
    player = ctx.obj["player"]
    click.echo(
        f"Switching stream for player ID {player.id} - {player.title} " f"to {title}..."
    )
    try:
        player.switch_stream(title)
    except URLError:
        raise click.UsageError("MultiViewer for F1 not found. Is the app "
                               "running?")
    except MultiViewerForF1Error as e:
        raise click.UsageError(f"MultiViewer for F1 error: {str(e)}")
    except Exception as e:
        raise click.UsageError(f"Unexpected error: {str(e)}")
    click.echo("Done.")


@cli.command(help="Run Model Context Protocol server.", name="mcp")
def run_mcp():
    mcp.run()



import json

from unittest import TestCase
from unittest.mock import patch

from click.testing import CliRunner
from mvf1.cmdline import cli

from urllib.error import URLError
from mvf1 import MultiViewerForF1Error


f = open('tests/players.json')
mock_players = json.load(f)
f.close()


def configure_mock_response(mock_urlopen, payload):
    if isinstance(payload, Exception) or isinstance(payload, list):
        mock_urlopen.side_effect = payload
    else:
        mock_urlopen.return_value = payload


class TestMultiViewerForF1CommandLineInterfacePlayers(TestCase):
    def setUp(self):
        self.runner = CliRunner()

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_ls(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                mock_players)

        response = self.runner.invoke(cli, ["players", "ls"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_ls_verbose(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                mock_players)

        response = self.runner.invoke(cli, ["players",
                                            "ls",
                                            "--verbose"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("streamData", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_ls_no_active_players(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {"data": {"players": []}})

        response = self.runner.invoke(cli, ["players", "ls"])

        self.assertEqual(response.exit_code, 2)
        self.assertIn("No active players", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_mute(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                mock_players)

        response = self.runner.invoke(cli, ["players", "mute"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_close(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                mock_players)

        response = self.runner.invoke(cli, ["players", "close"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_pause(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                mock_players)

        response = self.runner.invoke(cli, ["players", "pause"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_pause_exceptions(self, mock_urlopen):
        side_effects = [
            mock_players,
            URLError("Whoopsie doodle!"),
            mock_players,
            MultiViewerForF1Error("Whoopsie doodle!"),
            mock_players,
            Exception("Whoopsie doodle!")
        ]

        configure_mock_response(mock_urlopen,
                                side_effects)

        for i in range(3):
            response = self.runner.invoke(cli, ["players", "pause"])

            self.assertEqual(response.exit_code, 2)
            self.assertIn("error", response.output.lower())

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_mute_exceptions(self, mock_urlopen):
        side_effects = [
            mock_players,
            URLError("Whoopsie doodle!"),
            mock_players,
            MultiViewerForF1Error("Whoopsie doodle!"),
            mock_players,
            Exception("Whoopsie doodle!")
        ]

        configure_mock_response(mock_urlopen,
                                side_effects)

        for i in range(3):
            response = self.runner.invoke(cli, ["players", "mute"])

            self.assertEqual(response.exit_code, 2)
            self.assertIn("error", response.output.lower())

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_close_exceptions(self, mock_urlopen):
        side_effects = [
            mock_players,
            URLError("Whoopsie doodle!"),
            mock_players,
            MultiViewerForF1Error("Whoopsie doodle!"),
            mock_players,
            Exception("Whoopsie doodle!")
        ]

        configure_mock_response(mock_urlopen,
                                side_effects)

        for i in range(3):
            response = self.runner.invoke(cli, ["players", "close"])

            self.assertEqual(response.exit_code, 2)
            self.assertIn("error", response.output.lower())

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_sync(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                mock_players)

        response = self.runner.invoke(cli, ["players", "sync"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("broadcast commentary", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_toggle_speedometers(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                mock_players)

        response = self.runner.invoke(cli, ["players", "toggle-speedometers"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("Toggling speedometer", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_toggle_speedometers_exceptions(self, mock_urlopen):
        side_effects = [
            mock_players,
            URLError("Whoopsie doodle!"),
            mock_players,
            MultiViewerForF1Error("Whoopsie doodle!"),
            mock_players,
            Exception("Whoopsie doodle!")
        ]

        configure_mock_response(mock_urlopen,
                                side_effects)

        for i in range(3):
            response = self.runner.invoke(cli, ["players", "toggle-speedometers"])

            self.assertEqual(response.exit_code, 2)
            self.assertIn("error", response.output.lower())

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_exceptions_gql_error(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                MultiViewerForF1Error("Whoopsie doodle!"))

        commands = [
            ["players", "ls"],
            ["players", "mute"],
            ["players", "close"],
            ["players", "pause"],
            ["players", "sync"],
            ["players", "toggle-speedometers"]
        ]

        for command in commands:
            response = self.runner.invoke(cli, command)

            self.assertEqual(response.exit_code, 2)
            self.assertIn("error", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_exceptions_not_running(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                URLError("Whoopsie doodle!"))

        commands = [
            ["players", "ls"],
            ["players", "mute"],
            ["players", "close"],
            ["players", "pause"],
            ["players", "sync"],
            ["players", "toggle-speedometers"]
        ]

        for command in commands:
            response = self.runner.invoke(cli, command)

            self.assertEqual(response.exit_code, 2)
            self.assertIn("not found", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players_exceptions_generic_error(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                Exception("Whoopsie doodle!"))

        commands = [
            ["players", "ls"],
            ["players", "mute"],
            ["players", "close"],
            ["players", "pause"],
            ["players", "sync"],
            ["players", "toggle-speedometers"]
        ]

        for command in commands:
            response = self.runner.invoke(cli, command)

            self.assertEqual(response.exit_code, 2)
            self.assertIn("Error", response.output)


class TestMultiViewerForF1CommandLineInterfacePlayer(TestCase):
    def setUp(self):
        self.runner = CliRunner()

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_query(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player", "--id", "3", "query"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_query_verbose(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player", "--id", "3", "query",
                                            "--verbose"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)
        self.assertIn("streamData", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_query_by_title(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                mock_players)

        response = self.runner.invoke(cli, ["player", "--title", "INTERNATIONAL", "query"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

        response = self.runner.invoke(cli, ["player", "--title", "YOOOO", "query"])

        self.assertEqual(response.exit_code, 2)
        self.assertIn("No player", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_query_exceptions(self, mock_urlopen):
        side_effects = [
            URLError("Whoopsie doodle!"),
            MultiViewerForF1Error("Whoopsie doodle!"),
            Exception("Whoopsie doodle!"),
            URLError("Whoopsie doodle!"),
            MultiViewerForF1Error("Whoopsie doodle!"),
            Exception("Whoopsie doodle!")
        ]

        configure_mock_response(mock_urlopen,
                                side_effects)

        commands = [
            ["player", "--id", "3", "query"],
            ["player", "--id", "3", "query"],
            ["player", "--id", "3", "query"],
            ["player", "--title", "INTERNATIONAL", "query"],
            ["player", "--title", "INTERNATIONAL", "query"],
            ["player", "--title", "INTERNATIONAL", "query"],
        ]

        for command in commands:
            response = self.runner.invoke(cli, command)

            self.assertEqual(response.exit_code, 2)
            self.assertIn("Error", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_close(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player", "--id", "3", "close"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_mute(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player", "--id", "3", "mute"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_always_on_top(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player",
                                            "--id",
                                            "3",
                                            "set-always-on-top"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_pause(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player", "--id", "3", "pause"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_sync(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player", "--id", "3", "sync"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_seek(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player",
                                            "--id", "3",
                                            "seek", "--relative", "200"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_seek_no_options(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player",
                                            "--id", "3",
                                            "seek"])

        self.assertEqual(response.exit_code, 2)
        self.assertIn("To seek", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_set_bounds(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player",
                                            "--id", "3",
                                            "set-bounds",
                                            "--x", "0",
                                            "--y", "0",
                                            "--width", "640",
                                            "--height", "480"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_set_volume(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player", "--id", "3",
                                            "set-volume", "--volume", "28"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_set_fullscreen(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player", "--id", "3",
                                            "set-fullscreen"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_set_speedometer_visibility(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.runner.invoke(cli, ["player", "--id", "3",
                                            "set-speedometer-visibility"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_switch_stream(self, mock_urlopen):
        side_effects = [
            {"data": {"player": mock_players['data']['players'][0]}},
            {'data': {'playerDelete': True}},
            {'data': {'playerCreate': 5}},
            {"data": {"player": mock_players['data']['players'][0]}}
        ]
        configure_mock_response(mock_urlopen,
                                side_effects)

        response = self.runner.invoke(cli, ["player", "--id", "4",
                                            "switch-stream", "--title",
                                            "INTERNATIONAL"])

        self.assertEqual(response.exit_code, 0)
        self.assertIn("INTERNATIONAL", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_exceptions_gql_error(self, mock_urlopen):
        side_effects = [
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            MultiViewerForF1Error("Whoopsie doodle!")
        ]
        configure_mock_response(mock_urlopen,
                                side_effects)

        commands = [
            ["player", "--id", "3", "mute"],
            ["player", "--id", "3", "close"],
            ["player", "--id", "3", "pause"],
            ["player", "--id", "3", "sync"],
            ["player", "--id", "3", "mute"],
            ["player", "--id", "3", "set-always-on-top"],
            ["player", "--id", "3", "set-fullscreen"],
            ["player", "--id", "3", "set-speedometer-visibility"],
            ["player", "--id", "3", "seek", "--relative", "300"],
            ["player", "--id", "3", "set-volume", "--volume", "28"],
            ["player", "--id", "3", "set-bounds", "--x", "0", "--y", "0"],
            ["player", "--id", "3", "switch-stream", "--title", "INTERNATIONAL"]
        ]

        for command in commands:
            response = self.runner.invoke(cli, command)

            self.assertEqual(response.exit_code, 2)
            self.assertIn("error", response.output)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_exceptions_not_running(self, mock_urlopen):
        side_effects = [
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            URLError("Whoopsie doodle!")
        ]
        configure_mock_response(mock_urlopen,
                                side_effects)

        commands = [
            ["player", "--id", "3", "mute"],
            ["player", "--id", "3", "close"],
            ["player", "--id", "3", "pause"],
            ["player", "--id", "3", "sync"],
            ["player", "--id", "3", "mute"],
            ["player", "--id", "3", "set-always-on-top"],
            ["player", "--id", "3", "set-fullscreen"],
            ["player", "--id", "3", "seek", "--relative", "300"],
            ["player", "--id", "3", "set-speedometer-visibility"],
            ["player", "--id", "3", "set-volume", "--volume", "28"],
            ["player", "--id", "3", "set-bounds", "--x", "0", "--y", "0"],
            ["player", "--id", "3", "switch-stream", "--title", "INTERNATIONAL"]
        ]

        for command in commands:
            response = self.runner.invoke(cli, command)

            self.assertEqual(response.exit_code, 2)
            self.assertIn("error", response.output.lower())

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_exceptions_generic_error(self, mock_urlopen):
        side_effects = [
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!"),
            {"data": {"player": mock_players['data']['players'][0]}},
            Exception("Whoopsie doodle!")
        ]
        configure_mock_response(mock_urlopen,
                                side_effects)

        commands = [
            ["player", "--id", "3", "mute"],
            ["player", "--id", "3", "close"],
            ["player", "--id", "3", "pause"],
            ["player", "--id", "3", "sync"],
            ["player", "--id", "3", "mute"],
            ["player", "--id", "3", "set-always-on-top"],
            ["player", "--id", "3", "set-fullscreen"],
            ["player", "--id", "3", "set-speedometer-visibility"],
            ["player", "--id", "3", "seek", "--relative", "300"],
            ["player", "--id", "3", "set-volume", "--volume", "28"],
            ["player", "--id", "3", "set-bounds", "--x", "0", "--y", "0"],
            ["player", "--id", "3", "switch-stream", "--title", "INTERNATIONAL"]
        ]

        for command in commands:
            response = self.runner.invoke(cli, command)

            self.assertEqual(response.exit_code, 2)
            self.assertIn("error", response.output.lower())



import sgqlc.types


mvf1_schema = sgqlc.types.Schema()


__docformat__ = 'markdown'


########################################################################
# Scalars and Enumerations
########################################################################
class AlwaysOnTopLevel(sgqlc.types.Enum):
    '''Level for always on top mode  See
    https://developer.apple.com/documentation/appkit/nswindow/level

    Enumeration Choices:

    * `FLOATING`None
    * `MAIN_MENU`None
    * `MODAL_PANEL`None
    * `NORMAL`None
    * `POP_UP_MENU`None
    * `SCREEN_SAVER`None
    * `STATUS`None
    * `TORN_OFF_MENU`None
    '''
    __schema__ = mvf1_schema
    __choices__ = ('FLOATING', 'MAIN_MENU', 'MODAL_PANEL', 'NORMAL', 'POP_UP_MENU', 'SCREEN_SAVER', 'STATUS', 'TORN_OFF_MENU')


class BigInt(sgqlc.types.Scalar):
    '''Integer represented as a base-10 string'''
    __schema__ = mvf1_schema


Boolean = sgqlc.types.Boolean

class DriverHeaderMode(sgqlc.types.Enum):
    '''The driver header mode for on-board players

    Enumeration Choices:

    * `DRIVER_HEADER`: Show the driver's headshot and TLA
    * `NONE`: Show nothing at all
    * `OBC_LIVE_TIMING`: Show live timing data for the driver (when
      live timing is available)
    '''
    __schema__ = mvf1_schema
    __choices__ = ('DRIVER_HEADER', 'NONE', 'OBC_LIVE_TIMING')


Float = sgqlc.types.Float

ID = sgqlc.types.ID

Int = sgqlc.types.Int

class JSONObject(sgqlc.types.Scalar):
    '''Object with arbitrary, untyped data'''
    __schema__ = mvf1_schema


class PlayerType(sgqlc.types.Enum):
    '''The player type

    Enumeration Choices:

    * `ADDITIONAL`: Additional stream, such as International, F1 Live,
      Data channel or Driver tracker
    * `OBC`: On-board camera, for drivers
    '''
    __schema__ = mvf1_schema
    __choices__ = ('ADDITIONAL', 'OBC')


String = sgqlc.types.String

class SubscriptionType(sgqlc.types.Enum):
    '''The type of subscription

    Enumeration Choices:

    * `F1TV_ACCESS`None
    * `F1TV_PRO`None
    * `F1_ACCESS`None
    '''
    __schema__ = mvf1_schema
    __choices__ = ('F1TV_ACCESS', 'F1TV_PRO', 'F1_ACCESS')



########################################################################
# Input Objects
########################################################################
class PlayerCreateInput(sgqlc.types.Input):
    '''The input for creating a player'''
    __schema__ = mvf1_schema
    __field_names__ = ('content_id', 'channel_id', 'driver_tla', 'driver_number', 'stream_title', 'bounds', 'fullscreen', 'always_on_top', 'maintain_aspect_ratio')
    content_id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='contentId')
    '''The contentId for the player'''

    channel_id = sgqlc.types.Field(Int, graphql_name='channelId')
    '''The channelId (if applicable) for this player'''

    driver_tla = sgqlc.types.Field(String, graphql_name='driverTla')
    '''The TLA (Three-Letter-Acronym) for the driver (if applicable) for
    this player  > Note: this can be used instead of channelId
    '''

    driver_number = sgqlc.types.Field(Int, graphql_name='driverNumber')
    '''The driver number for the driver (if applicable) for this player
    > Note: this can be used instead of channelId
    '''

    stream_title = sgqlc.types.Field(String, graphql_name='streamTitle')
    '''The title of the stream (if applicable) for this player  > Note:
    this can be used instead of channelId
    '''

    bounds = sgqlc.types.Field('RectangleInput', graphql_name='bounds')
    '''The bounds for this window'''

    fullscreen = sgqlc.types.Field(Boolean, graphql_name='fullscreen')
    '''Whether the player should be fullscreen or not'''

    always_on_top = sgqlc.types.Field(Boolean, graphql_name='alwaysOnTop')
    '''Whether the player should be always on top or not'''

    maintain_aspect_ratio = sgqlc.types.Field(Boolean, graphql_name='maintainAspectRatio')
    '''Whether the player should maintain aspect ratio or not'''



class RectangleInput(sgqlc.types.Input):
    '''A rectangle input, with the dimensions and position'''
    __schema__ = mvf1_schema
    __field_names__ = ('x', 'y', 'width', 'height')
    x = sgqlc.types.Field(Int, graphql_name='x')
    '''The x position of the rectangle'''

    y = sgqlc.types.Field(Int, graphql_name='y')
    '''The y position of the rectangle'''

    width = sgqlc.types.Field(Int, graphql_name='width')
    '''The width of the rectangle'''

    height = sgqlc.types.Field(Int, graphql_name='height')
    '''The height of the rectangle'''




########################################################################
# Output Objects and Interfaces
########################################################################
class F1LiveTimingClock(sgqlc.types.Type):
    '''Clock information for the live timing'''
    __schema__ = mvf1_schema
    __field_names__ = ('paused', 'system_time', 'track_time', 'live_timing_start_time')
    paused = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='paused')
    '''Whether or not the clock is paused'''

    system_time = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='systemTime')
    '''The timestamp on the host computer that represents the `trackTime`'''

    track_time = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='trackTime')
    '''The timestamp at the track, at the time on the host computer'''

    live_timing_start_time = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='liveTimingStartTime')
    '''Live timing start time, indicating the first recorded event's
    timestamp (track time)
    '''



class F1LiveTimingState(sgqlc.types.Type):
    '''F1 live timing state'''
    __schema__ = mvf1_schema
    __field_names__ = ('archive_status', 'audio_streams', 'car_data', 'championship_prediction', 'content_streams', 'driver_list', 'extrapolated_clock', 'heartbeat', 'lap_count', 'lap_series', 'pit_lane_time_collection', 'position', 'race_control_messages', 'session_data', 'session_info', 'session_status', 'team_radio', 'timing_app_data', 'timing_data', 'timing_stats', 'top_three', 'track_status', 'weather_data', 'weather_data_series')
    archive_status = sgqlc.types.Field(JSONObject, graphql_name='ArchiveStatus')
    '''The status of the live timing archive, used for replay live timing'''

    audio_streams = sgqlc.types.Field(JSONObject, graphql_name='AudioStreams')
    '''Available audio commentary streams'''

    car_data = sgqlc.types.Field(JSONObject, graphql_name='CarData')
    '''Car telemetry data, speed, throttle, brake, gear, RPM, DRS'''

    championship_prediction = sgqlc.types.Field(JSONObject, graphql_name='ChampionshipPrediction')
    '''Live championship standings, available for sessions where points
    are being awarded
    '''

    content_streams = sgqlc.types.Field(JSONObject, graphql_name='ContentStreams')
    '''Text-based content streams (via ScribbleLive) and audio commentary'''

    driver_list = sgqlc.types.Field(JSONObject, graphql_name='DriverList')
    '''List of all participating drivers with basic information'''

    extrapolated_clock = sgqlc.types.Field(JSONObject, graphql_name='ExtrapolatedClock')
    '''Extrapolated session clock  The extrapolated session clock returns
    the time in UTC (via the `Utc` field) and the remaining time for
    this session (via the `Extrapolating` field), and if the clock is
    still extrapolating (via the `Extrapolating` field). You can use
    this data, along with the `liveTimingClock` field on the root
    query, to calculate the time remaining in the session.
    '''

    heartbeat = sgqlc.types.Field(JSONObject, graphql_name='Heartbeat')
    '''Heartbeats from the live timing server'''

    lap_count = sgqlc.types.Field(JSONObject, graphql_name='LapCount')
    '''Lap count, for sessions with a schedule number of laps'''

    lap_series = sgqlc.types.Field(JSONObject, graphql_name='LapSeries')
    '''Time-series data per lap for each driver's position during the
    session
    '''

    pit_lane_time_collection = sgqlc.types.Field(JSONObject, graphql_name='PitLaneTimeCollection')
    '''Current or recent pit-stop timing data'''

    position = sgqlc.types.Field(JSONObject, graphql_name='Position')
    '''GPS positioning for cars and safety cars'''

    race_control_messages = sgqlc.types.Field(JSONObject, graphql_name='RaceControlMessages')
    '''Messages sent by FIA race control'''

    session_data = sgqlc.types.Field(JSONObject, graphql_name='SessionData')
    '''Simple session data, containing session status and start times for
    multi-part sessions (like Qualifying)
    '''

    session_info = sgqlc.types.Field(JSONObject, graphql_name='SessionInfo')
    '''Session information, including session type, circuit, official
    names and the scheduled start time
    '''

    session_status = sgqlc.types.Field(JSONObject, graphql_name='SessionStatus')
    '''The current session status (for which a time-series is available
    in the `SessionData` topic)
    '''

    team_radio = sgqlc.types.Field(JSONObject, graphql_name='TeamRadio')
    '''Team radio message captures'''

    timing_app_data = sgqlc.types.Field(JSONObject, graphql_name='TimingAppData')
    '''Timing data for tyre stints, grid start positions, including tyre
    compound, best time set on it and the amount of laps run on it
    '''

    timing_data = sgqlc.types.Field(JSONObject, graphql_name='TimingData')
    '''Most timing data available, including lap times, sector times,
    intervals/gaps, pitstop status, speed traps and number of laps run
    '''

    timing_stats = sgqlc.types.Field(JSONObject, graphql_name='TimingStats')
    '''Statistics for the current session, best lap times, best sectors,
    best speeds
    '''

    top_three = sgqlc.types.Field(JSONObject, graphql_name='TopThree')
    '''Basic information about the top-three drivers in this session'''

    track_status = sgqlc.types.Field(JSONObject, graphql_name='TrackStatus')
    '''The track status, which changes when yellow or red flags are out'''

    weather_data = sgqlc.types.Field(JSONObject, graphql_name='WeatherData')
    '''Current weather data'''

    weather_data_series = sgqlc.types.Field(JSONObject, graphql_name='WeatherDataSeries')
    '''Historic weather data, for the current session'''



class FIAWECLiveTimingState(sgqlc.types.Type):
    '''FIA WEC live timing state'''
    __schema__ = mvf1_schema
    __field_names__ = ('entries', 'referential', 'params', 'flags', 'best_sectors', 'race_control', 'laps', 'stints')
    entries = sgqlc.types.Field(JSONObject, graphql_name='entries')
    '''The race entries, which are the cars competing, including the
    drivers of that car, and other information
    '''

    referential = sgqlc.types.Field(JSONObject, graphql_name='referential')
    '''Referential data, which includes the entries, driver data,
    category data, information about the sessions and the race/brands
    '''

    params = sgqlc.types.Field(JSONObject, graphql_name='params')
    '''Information related to the current session, and track data such as
    weather, flag state, etc.
    '''

    flags = sgqlc.types.Field(JSONObject, graphql_name='flags')
    '''Any flags that are currently active'''

    best_sectors = sgqlc.types.Field(JSONObject, graphql_name='best_sectors')
    '''Best sectors overall'''

    race_control = sgqlc.types.Field(JSONObject, graphql_name='race_control')
    '''Race control messages'''

    laps = sgqlc.types.Field(JSONObject, graphql_name='laps')
    '''Lap-by-lap data for each car'''

    stints = sgqlc.types.Field(JSONObject, graphql_name='stints')
    '''Stint data for each car'''



class LiveTimingClock(sgqlc.types.Type):
    '''Clock information for the live timing'''
    __schema__ = mvf1_schema
    __field_names__ = ('paused', 'system_time', 'track_time', 'live_timing_start_time')
    paused = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='paused')
    '''Whether or not the clock is paused'''

    system_time = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='systemTime')
    '''The timestamp on the host computer that represents the `trackTime`'''

    track_time = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='trackTime')
    '''The timestamp at the track, at the time on the host computer'''

    live_timing_start_time = sgqlc.types.Field(sgqlc.types.non_null(BigInt), graphql_name='liveTimingStartTime')
    '''Live timing start time, indicating the first recorded event's
    timestamp (track time)
    '''



class LiveTimingState(sgqlc.types.Type):
    '''F1 live timing state'''
    __schema__ = mvf1_schema
    __field_names__ = ('archive_status', 'audio_streams', 'car_data', 'championship_prediction', 'content_streams', 'driver_list', 'extrapolated_clock', 'heartbeat', 'lap_count', 'lap_series', 'pit_lane_time_collection', 'position', 'race_control_messages', 'session_data', 'session_info', 'session_status', 'team_radio', 'timing_app_data', 'timing_data', 'timing_stats', 'top_three', 'track_status', 'weather_data', 'weather_data_series')
    archive_status = sgqlc.types.Field(JSONObject, graphql_name='ArchiveStatus')
    '''The status of the live timing archive, used for replay live timing'''

    audio_streams = sgqlc.types.Field(JSONObject, graphql_name='AudioStreams')
    '''Available audio commentary streams'''

    car_data = sgqlc.types.Field(JSONObject, graphql_name='CarData')
    '''Car telemetry data, speed, throttle, brake, gear, RPM, DRS'''

    championship_prediction = sgqlc.types.Field(JSONObject, graphql_name='ChampionshipPrediction')
    '''Live championship standings, available for sessions where points
    are being awarded
    '''

    content_streams = sgqlc.types.Field(JSONObject, graphql_name='ContentStreams')
    '''Text-based content streams (via ScribbleLive) and audio commentary'''

    driver_list = sgqlc.types.Field(JSONObject, graphql_name='DriverList')
    '''List of all participating drivers with basic information'''

    extrapolated_clock = sgqlc.types.Field(JSONObject, graphql_name='ExtrapolatedClock')
    '''Extrapolated session clock  The extrapolated session clock returns
    the time in UTC (via the `Utc` field) and the remaining time for
    this session (via the `Extrapolating` field), and if the clock is
    still extrapolating (via the `Extrapolating` field). You can use
    this data, along with the `liveTimingClock` field on the root
    query, to calculate the time remaining in the session.
    '''

    heartbeat = sgqlc.types.Field(JSONObject, graphql_name='Heartbeat')
    '''Heartbeats from the live timing server'''

    lap_count = sgqlc.types.Field(JSONObject, graphql_name='LapCount')
    '''Lap count, for sessions with a schedule number of laps'''

    lap_series = sgqlc.types.Field(JSONObject, graphql_name='LapSeries')
    '''Time-series data per lap for each driver's position during the
    session
    '''

    pit_lane_time_collection = sgqlc.types.Field(JSONObject, graphql_name='PitLaneTimeCollection')
    '''Current or recent pit-stop timing data'''

    position = sgqlc.types.Field(JSONObject, graphql_name='Position')
    '''GPS positioning for cars and safety cars'''

    race_control_messages = sgqlc.types.Field(JSONObject, graphql_name='RaceControlMessages')
    '''Messages sent by FIA race control'''

    session_data = sgqlc.types.Field(JSONObject, graphql_name='SessionData')
    '''Simple session data, containing session status and start times for
    multi-part sessions (like Qualifying)
    '''

    session_info = sgqlc.types.Field(JSONObject, graphql_name='SessionInfo')
    '''Session information, including session type, circuit, official
    names and the scheduled start time
    '''

    session_status = sgqlc.types.Field(JSONObject, graphql_name='SessionStatus')
    '''The current session status (for which a time-series is available
    in the `SessionData` topic)
    '''

    team_radio = sgqlc.types.Field(JSONObject, graphql_name='TeamRadio')
    '''Team radio message captures'''

    timing_app_data = sgqlc.types.Field(JSONObject, graphql_name='TimingAppData')
    '''Timing data for tyre stints, grid start positions, including tyre
    compound, best time set on it and the amount of laps run on it
    '''

    timing_data = sgqlc.types.Field(JSONObject, graphql_name='TimingData')
    '''Most timing data available, including lap times, sector times,
    intervals/gaps, pitstop status, speed traps and number of laps run
    '''

    timing_stats = sgqlc.types.Field(JSONObject, graphql_name='TimingStats')
    '''Statistics for the current session, best lap times, best sectors,
    best speeds
    '''

    top_three = sgqlc.types.Field(JSONObject, graphql_name='TopThree')
    '''Basic information about the top-three drivers in this session'''

    track_status = sgqlc.types.Field(JSONObject, graphql_name='TrackStatus')
    '''The track status, which changes when yellow or red flags are out'''

    weather_data = sgqlc.types.Field(JSONObject, graphql_name='WeatherData')
    '''Current weather data'''

    weather_data_series = sgqlc.types.Field(JSONObject, graphql_name='WeatherDataSeries')
    '''Historic weather data, for the current session'''



class Mutation(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('version', 'player_create', 'player_delete', 'player_set_fullscreen', 'player_set_always_on_top', 'player_set_bounds', 'player_set_volume', 'player_set_paused', 'player_set_muted', 'player_seek_to', 'player_sync', 'player_set_speedometer_visibility', 'player_set_driver_header_mode')
    version = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='version')
    '''Get the version of the app'''

    player_create = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='playerCreate', args=sgqlc.types.ArgDict((
        ('input', sgqlc.types.Arg(sgqlc.types.non_null(PlayerCreateInput), graphql_name='input', default=None)),
))
    )
    '''Create a new player  Returns the ID of the created player

    Arguments:

    * `input` (`PlayerCreateInput!`): Input object for this player
    '''

    player_delete = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerDelete', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
))
    )
    '''Close a player

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    '''

    player_set_fullscreen = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSetFullscreen', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('fullscreen', sgqlc.types.Arg(Boolean, graphql_name='fullscreen', default=None)),
))
    )
    '''Control the fullscreen state of a player  Returns the new
    fullscreen state

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    * `fullscreen` (`Boolean`): Whether the player should be
      fullscreen or not, if not set the player will toggle fullscreen
    '''

    player_set_always_on_top = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSetAlwaysOnTop', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('always_on_top', sgqlc.types.Arg(Boolean, graphql_name='alwaysOnTop', default=None)),
        ('level', sgqlc.types.Arg(AlwaysOnTopLevel, graphql_name='level', default=None)),
))
    )
    '''Control the alwaysOnTop state of a player  Returns the new
    alwaysOnTop state

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    * `always_on_top` (`Boolean`): Whether the player should be
      alwaysOnTop or not, if not set the player will toggle
      alwaysOnTop
    * `level` (`AlwaysOnTopLevel`): The level of alwaysOnTop, defaults
      to "floating"
    '''

    player_set_bounds = sgqlc.types.Field(sgqlc.types.non_null('Rectangle'), graphql_name='playerSetBounds', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('bounds', sgqlc.types.Arg(sgqlc.types.non_null(RectangleInput), graphql_name='bounds', default=None)),
))
    )
    '''Set the bounds of a player

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    * `bounds` (`RectangleInput!`): The new bounds of the player, any
      values that are not specified will remain unchanged
    '''

    player_set_volume = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='playerSetVolume', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('volume', sgqlc.types.Arg(sgqlc.types.non_null(Float), graphql_name='volume', default=None)),
))
    )
    '''Set volume of a player

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    * `volume` (`Float!`): The volume of the player (0-100)
    '''

    player_set_paused = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSetPaused', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('paused', sgqlc.types.Arg(Boolean, graphql_name='paused', default=None)),
))
    )
    '''Play/pause a player  Returns the new paused state

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    * `paused` (`Boolean`): Whether the player should be paused or
      not, if not set the player will toggle play/paused state
    '''

    player_set_muted = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSetMuted', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('muted', sgqlc.types.Arg(Boolean, graphql_name='muted', default=None)),
))
    )
    '''Mute/unmute a player  Returns the new muted state

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    * `muted` (`Boolean`): Whether the player should be muted or not,
      if not set the player will toggle muted/unmuted state
    '''

    player_seek_to = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='playerSeekTo', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('absolute', sgqlc.types.Arg(Float, graphql_name='absolute', default=None)),
        ('relative', sgqlc.types.Arg(Float, graphql_name='relative', default=None)),
))
    )
    '''Seek a player  Requires either `absolute` or `relative` to be set
    > Note: This doesn't work for live players  Returns the seeked
    time

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    * `absolute` (`Float`): The timestamp to seek to
    * `relative` (`Float`): The relative amount to seek, negative
      values seek backwards and positive values seek forwards
    '''

    player_sync = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSync', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
))
    )
    '''Sync other players to a player  Returns `true` if the sync was
    successful

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    '''

    player_set_speedometer_visibility = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='playerSetSpeedometerVisibility', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('visible', sgqlc.types.Arg(Boolean, graphql_name='visible', default=None)),
))
    )
    '''Control the visibility of the driver speedometer

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    * `visible` (`Boolean`): The visibility of the speedometer, if not
      set the speedometer will toggle visibility
    '''

    player_set_driver_header_mode = sgqlc.types.Field(sgqlc.types.non_null(DriverHeaderMode), graphql_name='playerSetDriverHeaderMode', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
        ('mode', sgqlc.types.Arg(sgqlc.types.non_null(DriverHeaderMode), graphql_name='mode', default=None)),
))
    )
    '''Set the driver header mode for on-board players

    Arguments:

    * `id` (`ID!`): The browserWindowId of the player
    * `mode` (`DriverHeaderMode!`): The driver header mode
    '''



class Player(sgqlc.types.Type):
    '''A player object'''
    __schema__ = mvf1_schema
    __field_names__ = ('id', 'type', 'state', 'driver_data', 'stream_data', 'bounds', 'fullscreen', 'always_on_top', 'maintain_aspect_ratio')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    '''The browserWindowId of the player'''

    type = sgqlc.types.Field(sgqlc.types.non_null(PlayerType), graphql_name='type')
    '''The player type'''

    state = sgqlc.types.Field('PlayerState', graphql_name='state')
    '''The player state'''

    driver_data = sgqlc.types.Field('PlayerDriverData', graphql_name='driverData')
    '''If this is a player of type OBC, this will contain the available
    driver data
    '''

    stream_data = sgqlc.types.Field('PlayerStreamData', graphql_name='streamData')
    '''The stream metadata for this player'''

    bounds = sgqlc.types.Field(sgqlc.types.non_null('Rectangle'), graphql_name='bounds')
    '''The bounds of the player'''

    fullscreen = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='fullscreen')
    '''Whether the player is fullscreen or not'''

    always_on_top = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='alwaysOnTop')
    '''Whether the player is always on top or not'''

    maintain_aspect_ratio = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='maintainAspectRatio')
    '''Whether the player maintains aspect ratio or not'''



class PlayerDriverData(sgqlc.types.Type):
    '''Driver metadata for a player'''
    __schema__ = mvf1_schema
    __field_names__ = ('driver_number', 'tla', 'first_name', 'last_name', 'team_name')
    driver_number = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='driverNumber')
    '''The driver's driver number'''

    tla = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='tla')
    '''The drivers TLA (Three-Letter-Acronym)'''

    first_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='firstName')
    '''The driver's first name'''

    last_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='lastName')
    '''The driver's last name'''

    team_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='teamName')
    '''The constructor's name'''



class PlayerState(sgqlc.types.Type):
    '''The player state'''
    __schema__ = mvf1_schema
    __field_names__ = ('ts', 'paused', 'muted', 'volume', 'live', 'current_time', 'interpolated_current_time')
    ts = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='ts')
    '''The UNIX timestamp when the player state was last updated'''

    paused = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='paused')
    '''Whether the player is paused or not'''

    muted = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='muted')
    '''Whether the player is muted or not'''

    volume = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='volume')
    '''The player's volume (0-100)'''

    live = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='live')
    '''Whether the player is live or not'''

    current_time = sgqlc.types.Field(Float, graphql_name='currentTime')
    '''The current time of the player, at the time it was last updated'''

    interpolated_current_time = sgqlc.types.Field(Float, graphql_name='interpolatedCurrentTime')
    '''The interpolated current time of the player'''



class PlayerStreamData(sgqlc.types.Type):
    '''Stream metadata for players'''
    __schema__ = mvf1_schema
    __field_names__ = ('content_id', 'meeting_key', 'session_key', 'channel_id', 'title')
    content_id = sgqlc.types.Field(ID, graphql_name='contentId')
    '''The contentId for this stream'''

    meeting_key = sgqlc.types.Field(String, graphql_name='meetingKey')
    '''The meeting key for this stream (if part of a meeting)'''

    session_key = sgqlc.types.Field(String, graphql_name='sessionKey')
    '''The session key for this stream (if part of a meeting)'''

    channel_id = sgqlc.types.Field(Int, graphql_name='channelId')
    '''If part of a content container with multiple additional streams,
    the ID of the channel
    '''

    title = sgqlc.types.Field(String, graphql_name='title')
    '''The stream's title'''



class Query(sgqlc.types.Type):
    __schema__ = mvf1_schema
    __field_names__ = ('version', 'system_info', 'players', 'player', 'live_timing_state', 'live_timing_clock', 'f1_live_timing_state', 'f1_live_timing_clock', 'fiawec_live_timing_state', 'active_subscriptions')
    version = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='version')
    '''Get the version of the app'''

    system_info = sgqlc.types.Field(sgqlc.types.non_null('SystemInfo'), graphql_name='systemInfo')
    '''Basic system information (platform and CPU architecture)'''

    players = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Player))), graphql_name='players')
    '''Get all open players'''

    player = sgqlc.types.Field(sgqlc.types.non_null(Player), graphql_name='player', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(ID), graphql_name='id', default=None)),
))
    )
    '''Get a player by ID

    Arguments:

    * `id` (`ID!`)None
    '''

    live_timing_state = sgqlc.types.Field(LiveTimingState, graphql_name='liveTimingState')
    '''Get the live timing state'''

    live_timing_clock = sgqlc.types.Field(LiveTimingClock, graphql_name='liveTimingClock')
    '''Get the clock information for the live timing (for session clock
    etc.)
    '''

    f1_live_timing_state = sgqlc.types.Field(F1LiveTimingState, graphql_name='f1LiveTimingState')
    '''Get the F1 live timing state'''

    f1_live_timing_clock = sgqlc.types.Field(F1LiveTimingClock, graphql_name='f1LiveTimingClock')
    '''Get the clock information for the F1 live timing (for session
    clock etc.)
    '''

    fiawec_live_timing_state = sgqlc.types.Field(FIAWECLiveTimingState, graphql_name='fiawecLiveTimingState')
    '''Get the FIA WEC live timing state'''

    active_subscriptions = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('Subscription'))), graphql_name='activeSubscriptions')
    '''Active subscriptions for the current user'''



class Rectangle(sgqlc.types.Type):
    '''A rectangle, with dimensions and position'''
    __schema__ = mvf1_schema
    __field_names__ = ('x', 'y', 'width', 'height')
    x = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='x')
    '''The x position of the rectangle'''

    y = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='y')
    '''The y position of the rectangle'''

    width = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='width')
    '''The width of the rectangle'''

    height = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='height')
    '''The height of the rectangle'''



class Subscription(sgqlc.types.Type):
    '''A subscription'''
    __schema__ = mvf1_schema
    __field_names__ = ('subscription_type', 'expires_at', 'signature')
    subscription_type = sgqlc.types.Field(sgqlc.types.non_null(SubscriptionType), graphql_name='subscriptionType')
    '''The type of subscription'''

    expires_at = sgqlc.types.Field(Int, graphql_name='expiresAt')
    '''The date the subscription expires (in seconds since the epoch)'''

    signature = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='signature')
    '''A JWT token that can be used to verify the subscription  ### To
    verify:  1. Retrieve the public key from
    https://multiviewer.app/.well-known/subscription-verification-
    public-key.pub 2. Verify the signature using the public key  ###
    Example (Node.js)  ```ts import { verify } from "jsonwebtoken";
    // The public key can be retrieved from
    https://multiviewer.app/.well-known/subscription-verification-
    public-key.pub const publicKey = "---BEGIN PUBLIC KEY---\n...\n---
    END PUBLIC KEY---";  const verified = verify(token, publicKey, {
    algorithms: ["RS256"],   issuer: "multiviewer.app", }); ```
    '''



class SystemInfo(sgqlc.types.Type):
    '''Basic system information'''
    __schema__ = mvf1_schema
    __field_names__ = ('platform', 'arch')
    platform = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='platform')
    '''The current platform (win32, darwin, linux)'''

    arch = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='arch')
    '''The current CPU architecture (x64, arm64, etc.)'''




########################################################################
# Unions
########################################################################

########################################################################
# Schema Entry Points
########################################################################
mvf1_schema.query_type = Query
mvf1_schema.mutation_type = Mutation
mvf1_schema.subscription_type = Subscription




import json
from copy import deepcopy

from unittest import TestCase
from unittest.mock import patch

from mvf1 import MultiViewerForF1
from mvf1 import MultiViewerForF1Error

from mvf1.mvf1_schema import mvf1_schema as schema

from sgqlc.operation import Operation

f = open('tests/players.json')
mock_players = json.load(f)
f.close()


def configure_mock_response(mock_urlopen, payload):
    if isinstance(payload, Exception):
        mock_urlopen.side_effect = payload
    else:
        mock_urlopen.return_value = payload


class TestMultiViewerForF1(TestCase):
    def setUp(self):
        self.remote = MultiViewerForF1()
        self.query = Operation(schema.Query)
        self.mutation = Operation(schema.Mutation)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_server_error(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'errors': [{'message': 'Whoopsie'}]})

        with self.assertRaises(MultiViewerForF1Error):
            self.remote.live_timing_clock

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_live_timing_clock(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {"data": {"liveTimingClock": None}})

        response = self.remote.live_timing_clock

        self.query.live_timing_clock()

        mock_urlopen.assert_called_once()
        self.assertIn("liveTimingClock", str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_f1_live_timing_clock(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {"data": {"liveTimingClock": None}})

        response = self.remote.f1_live_timing_clock

        self.query.live_timing_clock()

        mock_urlopen.assert_called_once()
        self.assertIn("liveTimingClock", str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_live_timing_state(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'liveTimingState': None}})

        response = self.remote.live_timing_state

        mock_urlopen.assert_called_once()

        self.assertIn("liveTimingState", str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_f1_live_timing_state(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'liveTimingState': None}})

        response = self.remote.f1_live_timing_state

        mock_urlopen.assert_called_once()

        self.assertIn("liveTimingState", str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_fiawec_live_timing_state(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'liveTimingState': None}})

        response = self.remote.fiawec_live_timing_state

        mock_urlopen.assert_called_once()

        self.assertIn("liveTimingState", str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_players(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                mock_players)

        response = self.remote.players

        mock_urlopen.assert_called_once()

        self.assertEqual(len(response), 2)
        self.assertEqual(response[0].title, "INTERNATIONAL")

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_system_info(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'systemInfo': {'platform': 'linux', 'arch': 'x64'}}})

        response = self.remote.system_info

        mock_urlopen.assert_called_once()

        self.assertIn('systemInfo', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_version(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'version': '1.12.6'}})

        response = self.remote.version

        mock_urlopen.assert_called_once()

        self.assertIn('version', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        response = self.remote.player(3)

        mock_urlopen.assert_called_once()

        self.assertEqual(response.title, "INTERNATIONAL")

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_create(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerCreate': '3'}})

        stream_data = mock_players['data']['players'][0]['streamData']

        response = self.remote.player_create(stream_data['contentId'])

        self.assertEqual(mock_urlopen.call_count, 1)

        self.assertIn("playerCreate", str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_create_with_bounds(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerCreate': '3'}})

        stream_data = mock_players['data']['players'][0]['streamData']

        response = self.remote.player_create(stream_data['contentId'],
                                             x=0, y=0, width=640, height=480)

        self.assertEqual(mock_urlopen.call_count, 1)

        self.assertIn("playerCreate", str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_sync_to_commentary(self, mock_urlopen):

        mock_urlopen.side_effect = [
            mock_players,
            {'data': {'playerSync': True}}
        ]

        response = self.remote.player_sync_to_commentary()

        self.assertEqual(mock_urlopen.call_count, 2)

        self.assertIn('Sync', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_sync_to_commentary_no_commentary(self, mock_urlopen):
        altered = deepcopy(mock_players)
        altered['data']['players'][0]['streamData']['title'] = 'NOPE'

        mock_urlopen.side_effect = [
            altered,
            {'data': {'playerSync': True}}
        ]

        response = self.remote.player_sync_to_commentary()

        self.assertEqual(mock_urlopen.call_count, 1)

        self.assertIn('No player', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_delete(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerDelete': True}})

        response = self.remote.player_delete(3)

        mock_urlopen.assert_called_once()

        self.assertIn('playerDelete', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_seek_to(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSeekTo': 0}})

        response = self.remote.player_seek_to(3, absolute=0)

        mock_urlopen.assert_called_once()

        self.assertIn('SeekTo', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_set_bounds(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetBounds': {'x': 0, 'y': 0, 'width': 720, 'height': 408}}})

        response = self.remote.player_set_bounds(3, x=0, y=0)

        mock_urlopen.assert_called_once()

        self.assertIn('SetBounds', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_set_always_on_stop(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerAlwaysOnTop': True}})

        response = self.remote.player_set_always_on_top(3)

        mock_urlopen.assert_called_once()

        self.assertIn('AlwaysOnTop', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_set_driver_header_mode(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetDriverHeaderMode': 'OBC_LIVE_TIMING'}})

        response = self.remote.player_set_driver_header_mode("OBC_LIVE_TIMING")

        mock_urlopen.assert_called_once()

        self.assertIn('OBC_LIVE_TIMING', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_player_set_driver_header_mode_wrong_input(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                ValueError("Naughty."))

        with self.assertRaises(MultiViewerForF1Error):
            self.remote.player_set_driver_header_mode("wat?")


class TestPlayer(TestCase):
    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def setUp(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'player':
                                          mock_players['data']['players'][0]}})

        self.remote = MultiViewerForF1()
        self.player = self.remote.player(3)

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_pause(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetPaused': True}})

        response = self.player.pause()

        mock_urlopen.assert_called_once()

        self.assertIn('SetPaused', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_set_volume(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetVolume': 3}})

        response = self.player.set_volume(3)

        mock_urlopen.assert_called_once()

        self.assertIn('SetVolume', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_muted(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetMuted': True}})

        response = self.player.mute()

        mock_urlopen.assert_called_once()

        self.assertIn('SetMuted', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_always_on_top(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetAlwaysOnTop': True}})

        response = self.player.set_always_on_top()

        mock_urlopen.assert_called_once()

        self.assertIn('AlwaysOnTop', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_set_fullscreen(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetFullscreen': True}})

        response = self.player.set_fullscreen()

        mock_urlopen.assert_called_once()

        self.assertIn('SetFullscreen', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_set_speedometer_visibility(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetSpeedometerVisibility': True}})

        response = self.player.set_speedometer_visibility()

        mock_urlopen.assert_called_once()

        self.assertIn('playerSetSpeedometerVisibility', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_set_driver_header_mode(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetDriverHeaderMode': 'OBC_LIVE_TIMING'}})

        response = self.player.set_driver_header_mode("OBC_LIVE_TIMING")

        mock_urlopen.assert_called_once()

        self.assertIn('OBC_LIVE_TIMING', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_sync(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSync': True}})

        response = self.player.sync()

        mock_urlopen.assert_called_once()

        self.assertIn('Sync', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_seek(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSeekTo': 0}})

        response = self.player.seek(absolute=0)

        mock_urlopen.assert_called_once()

        self.assertIn('SeekTo', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_delete(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerDelete': True}})

        response = self.player.delete()

        mock_urlopen.assert_called_once()

        self.assertIn('Delete', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_set_bounds(self, mock_urlopen):
        configure_mock_response(mock_urlopen,
                                {'data': {'playerSetBounds': True}})

        response = self.player.set_bounds(x=0, y=0)

        mock_urlopen.assert_called_once()

        self.assertIn('SetBounds', str(response))

    @patch('sgqlc.endpoint.http.HTTPEndpoint.__call__')
    def test_switch_stream(self, mock_urlopen):

        mock_urlopen.side_effect = [
            {'data': {'playerDelete': True}},
            {'data': {'playerCreate': '3'}}
        ]

        response = self.player.switch_stream('INTERNATIONAL')

        self.assertEqual(mock_urlopen.call_count, 2)

        self.assertIn("playerCreate", str(response))



import logging
from typing import Optional

from sgqlc.operation import Operation
from sgqlc.endpoint.http import HTTPEndpoint

from .mvf1_schema import mvf1_schema as schema
from .mvf1_schema import PlayerCreateInput
from .mvf1_schema import RectangleInput


class MultiViewerForF1(object):
    """
    A class to control video players for MultiViewerForF1, the best way to
    watch Formula 1.

    To download the app, visit their website at https://multiviewer.app/.

    Parameters
    ----------
    uri: str, optional
    Uri to control the MultiViewerForF1 install.
    Defaults to http://localhost:101010/api/graphql

    Attributes
    ----------
    endpoint: HTTPEndpoint
        GraphQL API Endpoint of MultiViewerForF1.

    """

    def __init__(self, uri="http://localhost:10101/api/graphql"):
        self.endpoint = HTTPEndpoint(uri)

    def perform_operation(self,
                          operation: Operation) -> dict:
        """
        Performs the GraphQL operation.

        Parameters
        ----------
        operation: Operation
            GraphQL Operation.

        Returns
        -------
        dict
            MultiViewerForF1 API Response.

        """
        response = self.endpoint(operation)

        if "errors" in response:
            error = response["errors"][0]
            raise MultiViewerForF1Error(f"{error['message']}")
        else:
            return response

    @property
    def live_timing_clock(self) -> dict:
        """
        Returns the time for an event when it is live.

        Returns
        -------
        dict
            Current time.

        """
        operation = Operation(schema.Query)
        operation.f1_live_timing_clock()

        return self.perform_operation(operation)

    @property
    def live_timing_state(self) -> dict:
        """
        Returns state of live timing at current time.

        Returns
        -------
        dict
            Current state.

        """
        operation = Operation(schema.Query)
        operation.f1_live_timing_state()

        return self.perform_operation(operation)

    @property
    def f1_live_timing_clock(self) -> dict:
        """
        Returns the time for an event when it is live.

        Returns
        -------
        dict
            Current time.

        """
        operation = Operation(schema.Query)
        operation.f1_live_timing_clock()

        return self.perform_operation(operation)

    @property
    def f1_live_timing_state(self) -> dict:
        """
        Returns state of live timing at current time.

        Returns
        -------
        dict
            Current state.

        """
        operation = Operation(schema.Query)
        operation.f1_live_timing_state()

        return self.perform_operation(operation)

    @property
    def fiawec_live_timing_state(self) -> dict:
        """
        Returns state of live timing at current time.

        Returns
        -------
        dict
            Current state.

        """
        operation = Operation(schema.Query)
        operation.fiawec_live_timing_state()

        return self.perform_operation(operation)

    @property
    def players(self) -> list:
        """
        Returns a list of active MultiViewerForF1 players.

        Returns
        -------
        list
            List of Player objects.

        """
        operation = Operation(schema.Query)
        operation.players()

        players_data = self.perform_operation(operation)

        players = []

        for player_data in players_data["data"]["players"]:
            players.append(Player(player_data))

        return players

    @property
    def system_info(self) -> dict:
        """
        Returns the system information.

        Returns
        -------
        dict
            System information.

        """
        operation = Operation(schema.Query)
        operation.system_info()

        return self.perform_operation(operation)

    @property
    def version(self) -> dict:
        """
        Returns the MultiViewerForF1 version.

        Returns
        -------
        dict
            Version information.

        """
        operation = Operation(schema.Query)
        operation.version()

        return self.perform_operation(operation)

    def player(self, id: int) -> "Player":
        """
        Returns the player with specific id.

        Parameters
        ----------
        id: int
            Id of player.

        Returns
        -------
        dict
            Player object.

        """
        operation = Operation(schema.Query)
        operation.player(id=id)

        player_data = self.perform_operation(operation)
        return Player(player_data["data"]["player"])

    def player_create(
        self,
        content_id: int,
        driver_tla: Optional[str] = None,
        driver_number: Optional[int] = None,
        stream_title: Optional[str] = None,
        x: Optional[int] = None,
        y: Optional[int] = None,
        width: Optional[int] = None,
        height: Optional[int] = None,
        fullscreen: Optional[bool] = False,
        always_on_top: Optional[bool] = False,
        maintain_aspect_ratio: Optional[bool] = True,
    ) -> dict:
        """
        Creates a new player.

        Parameters
        ----------
        content_id: int
            Content Id.

        driver_tla: str, optional
            Driver three letter acronym (e.g. 'PER' or 'HAM').

        driver_number: int, optional
            Driver Number.

        stream_title: str, optional
            Stream Title.

        x: int, optional
            X coordinate of player's top left corner.

        y: int, optional
            Y coordinate of player's top left corner.

        width: int, optional
            Width of player.

        height: int, optional
            Height of player.

        fullscreen: bool, optional
            Fullscreen status.

        always_on_top: bool, optional
            Always on top status.

        maintain_aspect_ratio: bool, optional
            Maintain aspect ratio status.

        Returns
        -------
        dict
            Dict with result of playerCreate operation. If successful, will
            include the `player_id`.

            Implementation note: we return a `player_id` instead of a `Player`
            object is that there is ~200-800 millisecond lag in between the
            invocation of `player_create` and the `player_id` being accessible
            via the MultiViewer For F1 GraphQL API.  Player creation is
            non-blocking and there is no callback when it is complete. As a
            consequence, we leave this as an implmentation decision for you.

            Most folks put a sleep in between player creation and player
            access.
        """
        operation = Operation(schema.Mutation)

        if x or y or width or height:
            bounds = RectangleInput(x=x, y=y, width=width, height=height)
        else:
            bounds = None

        player = PlayerCreateInput(
            content_id=content_id,
            driver_tla=driver_tla,
            driver_number=driver_number,
            stream_title=stream_title,
            bounds=bounds,
            fullscreen=fullscreen,
            always_on_top=always_on_top,
            maintain_aspect_ratio=maintain_aspect_ratio,
        )

        operation.player_create(input=player)

        player_data = self.perform_operation(operation)

        return player_data

    def player_delete(self, id: int) -> dict:
        """
        Deletes a player.

        Parameters
        ----------
        id: int
            Id of player.

        Returns
        -------
        dict
            Deletion response.

        """
        operation = Operation(schema.Mutation)
        operation.player_delete(id=id)

        return self.perform_operation(operation)

    def player_seek_to(
        self, id: int, absolute: Optional[int] = None, relative: Optional[int] = None
    ) -> dict:
        """
        Seeks to a specific position.

        Parameters
        ----------
        id: int
            Id of player.

        absolute: int, optional
            Absolute position.

        relative: int, optional
            Relative position.

        Returns
        -------
        dict
            Seek response.

        """
        operation = Operation(schema.Mutation)
        operation.player_seek_to(id=id, absolute=absolute, relative=relative)

        return self.perform_operation(operation)

    def player_set_bounds(
        self,
        id: int,
        x: Optional[int] = None,
        y: Optional[int] = None,
        width: Optional[int] = None,
        height: Optional[int] = None,
    ) -> dict:
        """
        Set the bounds of a player.

        Parameters
        ----------
        id: int
            Id of player.

        x: int, optional
            X coordinate of player's top left corner.

        y: int, optional
            Y coordinate of player's top left corner.

        width: int, optional
            Width of player.

        height: int, optional
            Height of player.

        Returns
        -------
        dict
            True if operation is successful.

        """
        operation = Operation(schema.Mutation)

        bounds = RectangleInput(x=x, y=y, width=width, height=height)

        operation.player_set_bounds(id=id, bounds=bounds)

        return self.perform_operation(operation)

    def player_set_volume(self, id: int, volume: int) -> dict:
        """
        Set the volume of a player.

        Parameters
        ----------
        id: int
            Id of player.

        volume: int
            Volume of player.

        Returns
        -------
        dict
            True if operation is successful.

        """
        operation = Operation(schema.Mutation)
        operation.player_set_volume(id=id, volume=volume)

        return self.perform_operation(operation)

    def player_set_paused(self, id: int, paused: Optional[bool] = None) -> dict:
        """
        Pauses/unpauses player or specifies pause state for player.

        Parameters
        ----------
        id: int
            Id of player.

        paused: bool, optional
            Desired pause state for player.

        Returns
        -------
        dict
            True if operation is successful.

        """
        operation = Operation(schema.Mutation)
        operation.player_set_paused(id=id, paused=paused)

        return self.perform_operation(operation)

    def player_set_fullscreen(self, id: int, fullscreen: Optional[bool] = None) -> dict:
        """
        Toggles fullscreen for a player or specifies fullscreen state for
        player.

        Parameters
        ----------
        id: int
            Id of player.

        fullscreen: bool, optional
            Desired fullscreen state of player.

        Returns
        -------
        dict
            True if operation is successful.

        """
        operation = Operation(schema.Mutation)
        operation.player_set_fullscreen(id=id, fullscreen=fullscreen)

        return self.perform_operation(operation)

    def player_set_muted(self, id: int, muted: Optional[bool] = None) -> dict:
        """
        Mutes/unmutes player or specifies muted state for player.

        Parameters
        ----------
        id: int
            Id of player.

        muted: bool, optional
            Desired muted state of player.

        Returns
        -------
        dict
            True if operation is successful.

        """
        operation = Operation(schema.Mutation)
        operation.player_set_muted(id=id, muted=muted)

        return self.perform_operation(operation)

    def player_set_speedometer_visibility(
        self, id: int, visible: Optional[bool] = None
    ) -> dict:
        """
        Makes speedometer overlay on player visible/invisible or specifies
        visibility.

        Parameters
        ----------
        id: int 
            Id of player.

        visible: bool, optional
            Visibility state of speedometer of player.

        Returns
        -------
        dict
            True if operation is successful.

        """
        operation = Operation(schema.Mutation)
        operation.player_set_speedometer_visibility(id=id, visible=visible)

        return self.perform_operation(operation)

    def player_set_driver_header_mode(self,
                                      id: int,
                                      mode: Optional[str] = None) -> dict:

        """
        Sets the overlay display for a driver stream.

        Parameters
        ----------
        id: int
            Id of player.

        mode: str, optional
            Desired overlay of the player - can be DRIVER_HEADER, NONE or
            OBC_LIVE_TIMING

        Returns
        -------
        dict
            True if operation is successful.

        """
        operation = Operation(schema.Mutation)

        operation.player_set_driver_header_mode(id=id, mode=mode)

        try:
            return self.perform_operation(operation)
        except ValueError as e:
            raise MultiViewerForF1Error(f"{e} - can be DRIVER_HEADER, NONE"
                                         " or OBC_LIVE_TIMING.")

    def player_set_always_on_top(
        self, id: int, always_on_top: Optional[bool] = None
    ) -> dict:
        """
        Sets player on/off always on top.

        Parameters
        ----------
        id: int 
            Id of player.

        always_on_top: bool, optional
            Is the player always on top?

        Returns
        -------
        dict
            True if operation is successful.

        """
        operation = Operation(schema.Mutation)
        operation.player_set_always_on_top(id=id, always_on_top=always_on_top) 

        return self.perform_operation(operation)

    def player_sync(self, id: int) -> dict:
        """
        Synchronizes all players to the timestamp of specified player.

        Parameters
        ----------
        id: int
            Id of player.

        Returns
        -------
        dict
            True if operation is successful.

        """
        operation = Operation(schema.Mutation)
        operation.player_sync(id=id)

        return self.perform_operation(operation)

    def player_sync_to_commentary(self) -> dict:
        """
        Synchronizes all players to the player with a broadcast commentary stream.

        Returns
        -------
        dict
            True if operation is successful.

        """
        for player in self.players:
            if (
                player.stream_data["title"] == "INTERNATIONAL"
                or player.stream_data["title"] == "F1 LIVE"
            ):
                return player.sync()

        return {"data": "No player has commentary."}


class Player(object):
    """
    Encapsulate the behaviour of Player.

    Attributes
    ----------
    id: int
        Player Id.
    state: str
        Player state.
    driver_data: dict
        Player driver_data.
    stream_data: dict
        Player stream_data.
    content_id: int
        Player F1TV Content ID.
    channel_id: int
        Player F1TV Channel ID.
    title: str
        Title of stream playing in player.
    bounds: dict
        Player bounds.
    x: float
        X coordinate of player's top left corner.
    y: float
        Y coordinate of player's top left corner.
    width: float
        Width of player.
    height: float
        Height of player.
    fullscreen: bool
        Is player fullscreen?
    always_on_top: bool
        Is player always on top?
    maintain_aspect_ratio: bool
        Does player maintain aspect ratio?
    remote: MultiViewerForF1
        Interface to control MultiViewerForF1.
    """

    def __init__(self, player_dict: dict):
        self.id = player_dict["id"]
        self.state = player_dict["state"]
        self.driver_data = player_dict["driverData"]
        self.stream_data = player_dict["streamData"]
        self.content_id = self.stream_data["contentId"]
        self.channel_id = self.stream_data["channelId"]
        self.title = self.stream_data["title"]
        self.bounds = player_dict["bounds"]
        self.x = self.bounds["x"]
        self.y = self.bounds["y"]
        self.width = self.bounds["width"]
        self.height = self.bounds["height"]
        self.fullscreen = player_dict["fullscreen"]
        self.always_on_top = player_dict["alwaysOnTop"]
        self.maintain_aspect_ratio = player_dict["maintainAspectRatio"]
        self.remote = MultiViewerForF1()

    def __repr__(self) -> str:
        """
        String representing the player object

        Returns
        -------
        str
            String representing the player object.

        """
        return f"{self.id}: {self.stream_data['title']}"

    def delete(self) -> dict:
        """
        Delete this player.

        Returns
        -------
        dict
            Response of deletion operation.
        """
        return self.remote.player_delete(self.id)

    def seek(
        self, absolute: Optional[int] = None, relative: Optional[int] = None
    ) -> dict:
        """
        Seeks this player's timestamp to an absolute or relative position.

        Parameters
        ----------
        absolute: float, optional
            Absolute value.
        relative: float, optional
            Relative value.

        Returns
        -------
        dict
            Server response of this player's seek operation.

        """
        return self.remote.player_seek_to(self.id, absolute=absolute, relative=relative)

    def set_bounds(
        self,
        x: Optional[int] = None,
        y: Optional[int] = None,
        width: Optional[int] = None,
        height: Optional[int] = None,
    ) -> dict:
        """
        Set bounds of this player.

        Parameters
        ----------
        x: int, optional
            X coordinate of this player's top left corner.
        y: int, optional
            Y coordinate of this player's top left corner.
        width: int, optional
            Width of player.
        height: int, optional
            Height of player.

        Returns
        -------
        dict
            Server response of setting this player's bounds.


        """
        return self.remote.player_set_bounds(
            self.id, x=x, y=y, width=width, height=height
        )

    def set_volume(self, volume: int) -> dict:
        """
        Set volume of this player.

        Parameters
        ----------
        volume: int, optional
            Desired volume of this player.

        Returns
        -------
        dict
            Server response of setting this player's volume.


        """
        return self.remote.player_set_volume(self.id, volume=volume)

    def pause(self, paused: Optional[bool] = None) -> dict:
        """
        Pauses/unpauses player or specifies pause state for player.

        Parameters
        ----------
        paused: bool, optional
            Desired pause state of this player.

        Returns
        -------
        dict
            Server response of this player's pause state.


        """
        return self.remote.player_set_paused(self.id, paused=paused)

    def set_fullscreen(self, fullscreen: Optional[bool] = None) -> dict:
        """
        Toggles fullscreen for a player or specifies fullscreen state for
          player.

        Parameters
        ----------
        fullscreen: bool, optional
            Desired fullscreen state of this player.

        Returns
        -------
        dict
            Server response of this player's fullscreen state.


        """
        return self.remote.player_set_fullscreen(self.id, fullscreen=fullscreen)

    def mute(self, muted: Optional[bool] = None) -> dict:
        """
        Mutes/unmutes player or specifies muted state for player.

        Parameters
        ----------
        muted: bool, optional
            Mute the player.

        Returns
        -------
        dict
            Server response of this player's muted state.


        """
        return self.remote.player_set_muted(self.id, muted=muted)

    def set_speedometer_visibility(self, visible: Optional[bool] = None) -> dict:
        """
        Makes speedometer overlay on this player visible/invisible or specifies
          visibility.

        Parameters
        ----------
        visible: bool, optional
            Visibility of speedometer overlay on this player.

        Returns
        -------
        dict
            Server response fo this player's speedometer visibility.


        """
        return self.remote.player_set_speedometer_visibility(self.id, visible=visible)

    def set_driver_header_mode(self, mode: Optional[str] = None) -> dict:
        """
        Sets the overlay display for a driver stream.

        Parameters
        ----------
        id: int
            Id of player.

        mode: str, optional
            Desired overlay of the player - can be DRIVER_HEADER, NONE or
            OBC_LIVE_TIMING

        Returns
        -------
        dict
           Server response to this player's driver header mode. 

        """

        return self.remote.player_set_driver_header_mode(self.id, mode=mode)

    def set_always_on_top(self, always_on_top: Optional[bool] = None) -> dict:
        """
        Turns player on/off always on top.

        Parameters
        ----------
        always_on_top: bool, optional
            Is the player always on top?

        Returns
        -------
        dict
            Server response fo this player's speedometer visibility.


        """
        return self.remote.player_set_always_on_top(self.id,
                                                    always_on_top=always_on_top)

    def sync(self) -> dict:
        """
        Synchronize all MultiViewerForF1 players to the timestamp of this
        player.

        Returns
        -------
        dict
            Server response of this player's sync operation.


        """
        return self.remote.player_sync(self.id)

    def switch_stream(self, title: str) -> dict:
        """
        Switch stream of this player.

        Parameters
        ----------
        title: str
            Title of stream feed (e.g. 'INTERNATIONAL' for Crofty or 'PER' for
            Checo).

        Returns
        -------
        dict 
            Dict with result of player_create operation.

            Please see the implementation note on 
            `MultiViewerForF1.player_create`.

        """
        self.delete()

        return self.remote.player_create(
            self.content_id,
            stream_title=title,
            x=self.x,
            y=self.y,
            width=self.width,
            height=self.height,
        )


class MultiViewerForF1Error(Exception):
    """
    Wrap errors from MultiViewerForF1 API.

    Attributes
    ----------
    message: str
        Error message.

    Methods
    -------
    __init__(self, message)
        Log the error.

    """

    def __init__(self, message: str):
        logging.error(f"ERROR: {message}")



from .mvf1 import MultiViewerForF1
from .mvf1 import MultiViewerForF1Error
from .mvf1 import Player
from .mcp import mcp

__title__ = "mvf1"
__version__ = "2.0.0"
__author__ = "Rob Spectre"
__license__ = "MIT"
__copyright__ = "Copyright 2025 Rob Spectre"
